import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_gauges/gauges.dart';

void main() => runApp(const MyApp());

class MyApp extends StatelessWidget {
  const MyApp({super.key});

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: const MyHomePage(title: 'Flutter Demo Home Page'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({Key? key, required this.title}) : super(key: key);

  final String title;

  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      body: Center(
        child: SfRadialGauge(
          axes: <RadialAxis>[
            RadialAxis(
                labelOffset: 15,
                axisLineStyle: const AxisLineStyle(
                    thicknessUnit: GaugeSizeUnit.factor,
                    thickness: 0.15,
                    color: Colors.blue),
                radiusFactor: 0.9,
                minimum: -5,
                maximum: 15,
                showTicks: true,
                showLastLabel: true,
                labelsPosition: ElementsPosition.outside,
                onCreateAxisRenderer: () {
                  final _CustomAxisRenderer renderer = _CustomAxisRenderer();
                  return renderer;
                },
                axisLabelStyle: const GaugeTextStyle(fontSize: 12),
                pointers: const <GaugePointer>[
                  NeedlePointer(
                    value: 10,
                    lengthUnit: GaugeSizeUnit.factor,
                    animationDuration: 1300,
                    needleStartWidth: 0,
                    needleEndWidth: 8,
                    needleLength: 0.5,
                    knobStyle: KnobStyle(
                      knobRadius: 0,
                    ),
                  ),
                ])
          ],
        ),
      ),
    );
  }
}

class _CustomAxisRenderer extends RadialAxisRenderer {
  _CustomAxisRenderer() : super();

  /// Generated the 9 non-linear interval labels from 0 to 150
  /// instead of actual generated labels.
  @override
  List<CircularAxisLabel> generateVisibleLabels() {
    final List<CircularAxisLabel> visibleLabels = <CircularAxisLabel>[];
    for (num i = 0; i < 8; i++) {
      final double value = _calculateLabelValue(i);
      final CircularAxisLabel label = CircularAxisLabel(
          axis.axisLabelStyle, value.toInt().toString(), i, false);
      label.value = value;
      visibleLabels.add(label);
    }

    return visibleLabels;
  }

  // Default sweep angle for the widget is 280 as the default value of start and end angle of the radial gauge is 130 and 50 respectively.
  // sweepAngle = 280;

  // To calculate the angle difference for the radial axis range scales to find the angle diff factor for value to factor conversion, you must divide the sweep angle by no. of range sections.
  // In your case, there are 8 non-linear axis labels and so the no. of scale range sections is 7. 
  // Note: you can change the no. of sections based on your requirement at your end.
  // angleDiff = 280 / 7;

  double angleDiffFactor = (280 / 7) / 280;


  // Returns the factor(0 to 1) from value to place the labels in an axis.
  @override
  double valueToFactor(double value) {
    if (value >= -5 && value <= 0) {
      return (((value + 5) * angleDiffFactor) / 5);
    } else if (value > 0 && value <= 2) {
      return (((value - 0) * angleDiffFactor) / (2 - 0)) + (1 * angleDiffFactor);
    } else if (value > 2 && value <= 4) {
      return (((value - 2) * angleDiffFactor) / (4 - 2)) + (2 * angleDiffFactor);
    } else if (value > 4 && value <= 6) {
      return (((value - 4) * angleDiffFactor) / (6 - 4)) + (3 * angleDiffFactor);
    } else if (value > 6 && value <= 8) {
      return (((value - 6) * angleDiffFactor) / (8 - 6)) + (4 * angleDiffFactor);
    } else if (value > 8 && value <= 10) {
      return (((value - 8) * angleDiffFactor) / (10 - 8)) + (5 * angleDiffFactor);
    } else if (value > 10 && value <= 15) {
      return (((value - 10) * angleDiffFactor) / (15 - 10)) + (6 * angleDiffFactor);
    } else {
      return 1;
    }
  }

  /// To return the label value based on interval
  double _calculateLabelValue(num value) {
    if (value == 0) {
      return -5;
    } else if (value == 1) {
      return 0;
    } else if (value == 2) {
      return 2;
    } else if (value == 3) {
      return 4;
    } else if (value == 4) {
      return 6;
    } else if (value == 5) {
      return 8;
    } else if (value == 6) {
      return 10;
    } else {
      return 15;
    }
  }
}
